package com.xxh.cloud.frontend.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.handler.MappedInterceptor;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.ArrayList;
import java.util.List;

@Configuration
@EnableSwagger2
@ComponentScan(basePackages = {"com.xxh"})
public class Swager2Config {

    @Value("${saas.swagger.basic.username:admin}")
    private String username;
    @Value("${saas.swagger.basic.password:admin}")
    private String password;

    /* 必须在此处配置拦截器,要不然拦不到swagger的静态资源 */
    @Bean
    @ConditionalOnProperty(name = "saas.swagger.basic.enable", havingValue = "true")
    public MappedInterceptor getMappedInterceptor() {
        return new MappedInterceptor(
                new String[]{"/swagger-ui.html","/swagger/**", "/swagger-resources","/swagger-resources/**"
                        ,"/webjars/**", "/v2/api-docs"},
                new SwaggerInterceptor("username", "password"));
    }

    @Bean
    public Docket createRestApi() {
        //添加head参数start
        List<Parameter> pars = new ArrayList<Parameter>();
        pars.add(new ParameterBuilder().name("token").description("令牌").modelRef(new ModelRef("string")).parameterType("header").required(false).build());
       //pars.add(new ParameterBuilder().name("saas").description("公司编号").modelRef(new ModelRef("string")).parameterType("header").required(false).build());
        pars.add(new ParameterBuilder().name("lang").description("语言").modelRef(new ModelRef("string")).parameterType("header").required(false).build());

        return new Docket(DocumentationType.SWAGGER_2)
                .apiInfo(apiInfo())
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.xxh"))
                .paths(PathSelectors.any())
                .build()
                .globalOperationParameters(pars);
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("API接口定义")
                .description("API接口定义说明")
                .version("1.0")
                .build();
    }
}
