package com.xxh.cloud.frontend.common.modules.his;


import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.xxh.cloud.frontend.common.modules.his.config.SysProjectConfig;
import com.xxh.cloud.frontend.common.modules.his.dto.GenerateSchemeDTO;
import com.xxh.cloud.frontend.common.modules.his.model.GenerateSchemeModel;
import com.xxh.cloud.frontend.common.modules.his.model.HisAccessTokenModel;
import com.xxh.cloud.frontend.common.modules.his.model.PatBindModel;
import com.xxh.cloud.frontend.common.modules.modules.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;


@Slf4j
@Component
public class HisApiHelper {

    // 获取accessToken
    private static String accessTokenUrl = "/api/customize/getAccessToken";

    // 获取关联就诊卡号列表
    private static String patInfoUrl = "/api/customize/getPatInfoByOpenId";

    // 获取schemeurl
    private static String schemeUrl = "/api/customize/generatescheme";


    public static Result<HisAccessTokenModel> getAccessToken(SysProjectConfig config) {
        String result = HttpRequest.post(config.getHost() + accessTokenUrl)
                .form("key", config.getKey())
                .execute().body();
//        String result = "{\"code\":0,\"msg\":\"success\",\"data\":{\"jsTicket\":\"LIKLckvwlJT9cWIhEQTwfGKqz2-w-FGuBew9PQLmE4N7XxJ4H0w_0zIJMm4juEJ2P_SRhJdawhjbHPYJ24OTHg\",\"accToken\":\"74_lxg_-TPLPV2kQGVoHXifIN7OpeHtf15B6FmO3zSb5naQpn5a7L5kusU2nHCEdmG4oP3utDVdwiFJEoiNRafg7aYBNDG7IVGwbmL3qEMUL9pdPitv8zrjJVfNnEoKXDgAEADQW\"}}";
        log.info("获取token----" + result);
        return new Result<>(result, HisAccessTokenModel.class);
    }

    public static PatBindModel getPatBind(SysProjectConfig config, String userId) {
        String result = HttpRequest.post(config.getHost() + patInfoUrl)
                .form("key", config.getKey())
                .form("userId", userId)
                .execute().body();
//        String result = "{\"code\":0,\"data\":[{\"birthday\":\"1983-02-20\",\"relationType\":5,\"patientMobile\":\"18621566608\",\"patientName\":\"李明\",\"idTypeName\":\"身份证\",\"patCardNo\":\"5001285739\",\"patientSex\":\"M\",\"idType\":1,\"patientId\":4799929934901673994,\"patHisNo\":\"5001285739\",\"patientType\":0,\"patCardType\":21,\"patInNo\":\"\",\"idNo\":\"310113198302200814\",\"extFields\":\"{\\\"qrCodeText\\\":\\\"417E7B2E83CD17B8651652AC54E98132F041659E17E9CC6C622B83A59C8B7BBB:1::3400A0006APPF0098\\\",\\\"healthCardId\\\":\\\"417E7B2E83CD17B8651652AC54E98132F041659E17E9CC6C622B83A59C8B7BBB\\\"}\",\"isDefault\":0,\"relationName\":\"他人\",\"patCardNoEncry\":\"gD0zGxPsj2LAgI/FroFFHQ==\",\"healthCardFlag\":2,\"patCardTypeName\":\"院内就诊卡\",\"bindStatus\":1},{\"birthday\":\"1985-10-23\",\"relationType\":5,\"patientMobile\":\"18095681616\",\"patientName\":\"王翀\",\"idTypeName\":\"身份证\",\"patCardNo\":\"5000359206\",\"patientSex\":\"M\",\"idType\":1,\"patientId\":4706068853493334017,\"patHisNo\":\"5000359206\",\"patientType\":0,\"patCardType\":21,\"patInNo\":\"\",\"idNo\":\"340621198510230332\",\"extFields\":\"{\\\"qrCodeText\\\":\\\"BD2B78EAB6C419CD88245783605501453FC701619CE78BE85177D2F17FAE52A5:1::3400A0006APPF0098\\\",\\\"healthCardId\\\":\\\"BD2B78EAB6C419CD88245783605501453FC701619CE78BE85177D2F17FAE52A5\\\"}\",\"isDefault\":1,\"relationName\":\"他人\",\"patCardNoEncry\":\"FShUj1EXvue8Uxl+Epw9nQ==\",\"healthCardFlag\":2,\"patCardTypeName\":\"院内就诊卡\",\"bindStatus\":1}]}";
        log.info("获取关联就诊卡号----" + result);
        return JSONObject.parseObject(result, PatBindModel.class);
    }

    public static Result<GenerateSchemeModel> generateScheme(SysProjectConfig config, GenerateSchemeDTO params) {
        String result = HttpRequest.post(config.getHost() + schemeUrl)
                .form("accessToken", params.getAccessToken())
                .form("deptCode", params.getDeptCode())
                .form("doctorCode", params.getDoctorCode())
                .execute().body();
//        String result = "{\"code\":0,\"msg\":\"success\",\"data\":{\"errcode\":0,\"errmsg\":\"ok\",\"openlink\":\"weixin://dl/business/?t=zPkEbEFpgae\"}}";
        log.info("获取schemeurl----" + result);
        return new Result<>(result, GenerateSchemeModel.class);
    }

    // 发送短信



    public static void main(String[] args) {
//        String bodyJson = JSONObject.toJSONString(new AccessTokenParams("GVbIW6PDULXEcEzq", "EFlt6A8rnzJC1bg8"));
//        Map<String, String> headers = new HashMap<>();
//        headers.put("token", "AUK_1697567919187300352");
//        String result = HttpRequest.post("https://dev1.cloud-seal.com/api/healthy-dev/h/mgr/custUser/queryByAll")
//                .headerMap(headers, true)
//                .body(bodyJson)
//                .execute().body();
//        System.out.println(result);
//
    }

}
