package com.xxh.cloud.frontend.custuser.web.controller;

import com.company.cloud.sdk.weixin.common.config.WxApiConfig;
import com.company.cloud.sdk.weixin.mini.model.Code2SessionResp;
import com.company.cloud.sdk.weixin.mp.model.Code2AccessTokenResp;
import com.company.cloud.sdk.weixin.mp.model.JsapiTicketResp;
import com.company.cloud.sdk.weixin.mp.model.RefreshTokenResp;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoResp;
import com.xxh.cloud.frontend.common.modules.his.config.SysProjectConfig;
import com.xxh.cloud.frontend.common.modules.his.model.HisAccessTokenModel;
import com.xxh.cloud.frontend.common.modules.modules.Result;
import com.xxh.cloud.frontend.common.modules.util.IpWhite;
import com.xxh.cloud.frontend.common.modules.weixin.ThirdWeixinAccessTokenService;
import com.xxh.cloud.frontend.common.modules.weixin.ThirdWeixinLoginMiniService;
import com.xxh.cloud.frontend.common.modules.weixin.ThirdWeixinLoginMpService;
import com.xxh.cloud.frontend.common.modules.weixin.ThirdWeixinPageAccessTokenService;
import com.xxh.cloud.frontend.common.modules.weixin.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@Slf4j
@RestController
@RequestMapping("/api/weixin")
@Api(tags = "调用微信服务接口")
public class CustUserWxLoginController {

    @Autowired
    private SysProjectConfig hisconfig;
    @Autowired
    private ThirdWeixinAccessTokenService thirdWeixinAccessTokenService;
    @Autowired
    private ThirdWeixinLoginMiniService thirdWeixinLoginMiniService;
    @Autowired
    private ThirdWeixinLoginMpService thirdWeixinLoginMpService;
    @Autowired
    private ThirdWeixinPageAccessTokenService thirdWeixinPageAccessTokenService;
    @Autowired
    private IpWhite ipWhite;

    // 不使用此接口，调用HIS那边的统一接口获取AccessToken
    @PostMapping("/getAccessToken")
    @ApiOperation(value = "获取微信的AccessToken", response = Result.class)
    public Result<HisAccessTokenModel> getAccessToken(@RequestBody WxApiConfig config) {
        // ip白名单限制
        ipWhite.validatorIpWhite();

        String accessToken = thirdWeixinAccessTokenService.getAccessToken(config);
        HisAccessTokenModel model = new HisAccessTokenModel();
        model.setAccToken(accessToken);
        return new Result<HisAccessTokenModel>().ok(model);
    }

    @PostMapping("/jsapiTicket")
    @ApiOperation(value = "用AccessToken查询JsapiTicket", response = Result.class)
    public Result<JsapiTicketResp> jsapiTicket(@RequestBody AccessTokenDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<JsapiTicketResp>().ok(thirdWeixinAccessTokenService.getJsapiToken(params.getAccessToken()));
    }

    @PostMapping("/mini/getLoginAuthToken")
    @ApiOperation(value = "小程序端-获取登录信息", response = Result.class)
    public Result<Code2SessionResp> getMiniLoginAuthToken(@RequestBody LoginAuthTokenDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<Code2SessionResp>().ok(
                thirdWeixinLoginMiniService.getLoginAuthToken(params, params.getCode()));
    }

    @PostMapping("/mini/getMobileByOpenId")
    @ApiOperation(value = "小程序端-获取手机号", response = Result.class)
    public Result<String> getMobileByOpenId(@RequestBody BindMobileByWxDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<String>().ok(thirdWeixinLoginMiniService.getMobileByOpenId(params));
    }

    @PostMapping("/mp/getLoginAuthToken")
    @ApiOperation(value = "公众号-获取登录信息", response = Result.class)
    public Result<Code2AccessTokenResp> getMpLoginAuthToken(@RequestBody LoginAuthTokenDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<Code2AccessTokenResp>().ok(
                thirdWeixinLoginMpService.getLoginAuthToken(params, params.getCode()));
    }

    @PostMapping("/getUnionid")
    @ApiOperation(value = "获取Unionid", response = Result.class)
    public Result<UserBasicInfoResp> getUnionid(@RequestBody UnionIdDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<UserBasicInfoResp>().ok(
                thirdWeixinLoginMpService.getUnionid(params.getOpenid(), params.getAccess_token()));
    }

    @PostMapping("/page/refreshToken")
    @ApiOperation(value = "刷新网页授权调用凭证", response = Result.class)
    public Result<RefreshTokenResp> refreshToken(@RequestBody RefreshTokenDTO params) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<RefreshTokenResp>().ok(
                thirdWeixinPageAccessTokenService.refresh(params.getAppId(), params.getRefresh_token()));
    }

    @GetMapping("/getUnionidTest")
    @ApiOperation(value = "获取Unionid", response = Result.class)
    public Result<UserBasicInfoResp> getUnionidTest(@RequestParam("accToken") String accToken) {
        // ip白名单限制
        ipWhite.validatorIpWhite();
        
        return new Result<UserBasicInfoResp>().ok(
                thirdWeixinLoginMpService.getUnionid("o6jK352u-eNoiEeQb2KsUKwAF8_c", accToken));
    }
}
