package com.xxh.cloud.frontend.custuser.web.controller;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.company.cloud.sdk.weixin.common.utils.AesUtils;
import com.company.cloud.sdk.weixin.common.utils.XmlUtils;
import com.company.cloud.sdk.weixin.open.model.ComponentReceiveEventFullModel;
import com.company.cloud.sdk.weixin.open.model.ComponentVerifyTicketModel;
import com.xxh.cloud.frontend.common.modules.his.config.SysProjectConfig;
import com.xxh.cloud.frontend.common.modules.modules.Result;
import com.xxh.cloud.frontend.common.modules.util.IpWhite;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping("/sys/healthy/wechat/third")
@Api(tags = "微信相关")
public class WechatThirdController {

    @Autowired
    private IpWhite ipWhite;
    @Autowired
    private SysProjectConfig sysProjectConfig;

    @Value("${wechat.open.appid}")
    private String appid;
    @Value("${wechat.open.token}")
    private String token;
    @Value("${wechat.open.encodingAesKey:}")
    private String encodingAesKey;

    // 开放平台事件处理url
    @Value("${wechat.open.componentEventDealUrl:}")
    private String componentEventDealUrl;
    // 开放平台component_verify_ticket事件处理url
    @Value("${wechat.open.componentVerifyTicketUrl:}")
    private String componentVerifyTicketUrl;


    @PostMapping("/componentVerifyTicket")
    @ApiOperation(value = "第三方开放平台获取验证票据", response = String.class)
    public String componentVerifyTicket(HttpServletRequest request, HttpServletResponse response, @RequestBody String componentVerifyTicketMsg) {
        String msgSignature = request.getParameter("msg_signature");
        String timestamp = request.getParameter("timestamp");
        String nonce = request.getParameter("nonce");
        String encryptType = request.getParameter("encrypt_type");
        log.info("url参数>>>>>> msgSignature={}, timestamp={}, nonce={}, encryptType={}", msgSignature, timestamp, nonce, encryptType);

        // 数据解密
        if (StrUtil.isNotBlank(encodingAesKey)) {
            try {
                componentVerifyTicketMsg = AesUtils.decryptMsg(token, appid, encodingAesKey, componentVerifyTicketMsg, msgSignature, timestamp, nonce);
            } catch (Exception e) {
                log.error("消息解密异常>>>>>> ", e.getMessage());
                e.printStackTrace();
            }
        }

        log.info("解密后数据>>>>>>> {}", componentVerifyTicketMsg);
        // 数据字符串转对象
        ComponentVerifyTicketModel componentVerifyTicketModel = (ComponentVerifyTicketModel) XmlUtils.convertXmlStrToObject(ComponentVerifyTicketModel.class, componentVerifyTicketMsg);
        log.info("字符串参数转对象>>>>>> {}", JSON.toJSONString(componentVerifyTicketModel));

        // TODO 保存componentVerifyTicket，并判断component_access_token是否过期，过期则重新获取
        // try {
        //     HttpUtil.post(componentVerifyTicketUrl, JSON.toJSONString(componentVerifyTicketModel));
        // } catch (Exception e) {
        //     log.info("保存componentVerifyTicket error >>>>>>", e.getMessage());
        //     e.printStackTrace();
        // }

        return "success";
    }

    // 第三方验证消息事件token，配置服务地址
    // @GetMapping("/third/passivemsg")
    // @ApiOperation(value = "被动回复消息验证", response = String.class)
    // public String thirdPassiveMsgToken(HttpServletRequest request, HttpServletResponse response) {
    //     String echostr = null;
    //     try {
    //         String signature = request.getParameter("signature");
    //         if (StrUtil.isNotBlank(signature)) {
    //             String timestamp = request.getParameter("timestamp");
    //             String nonce = request.getParameter("nonce");
    //             String encryptType = request.getParameter("encrypt_type");
    //             echostr = request.getParameter("echostr");
    //             log.info("/sys/healthy/wechat/msg/third/passivemsg:params >>>>>> signature={}, echostr={}, timestamp={}, nonce={}, encryptType={}", signature, echostr, timestamp, nonce, encryptType);
    //             if (WxVerificationUtils.checkSignature(token, signature, timestamp, nonce)) {
    //                 log.info("验证通过");
    //                 return echostr;
    //             }
    //         }
    //     } catch (Exception e) {
    //         log.error("校验异常>>>>>> {}", e.getMessage());
    //         e.printStackTrace();
    //     }
    //     log.info("验证失败");
    //     return "error";
    // }

    // 第三方授权事件接收，配置服务地址
    @PostMapping("/{appid}/passivemsg")
    @ApiOperation(value = "第三方授权事件接收", response = String.class)
    public String thirdPassiveMsgToken(HttpServletRequest request, HttpServletResponse response, @PathVariable("appid") String mpAppid, @RequestBody String passiveReplyMsg) {
        log.info("/sys/healthy/wechat/msg/third/passivemsg:passiveReplyMsg >>>>>> {}, {}", mpAppid, passiveReplyMsg);

        String timestamp = request.getParameter("timestamp");
        String nonce = request.getParameter("nonce");
        String encryptType = request.getParameter("encrypt_type");
        String msgSignature = request.getParameter("msg_signature");
        log.info("url参数>>>>>> msgSignature={}, timestamp={}, nonce={}, encryptType={}", msgSignature, timestamp, nonce, encryptType);

        // 数据解密
        if (StrUtil.isNotBlank(encodingAesKey)) {
            try {
                passiveReplyMsg = AesUtils.decryptMsg(token, appid, encodingAesKey, passiveReplyMsg, msgSignature, timestamp, nonce);
            } catch (Exception e) {
                log.error("消息解密异常>>>>>> ", e.getMessage());
                e.printStackTrace();
            }
        }

        log.info("解密后数据>>>>>>> {}", passiveReplyMsg);

        // 数据字符串转对象
        ComponentReceiveEventFullModel componentReceiveEventFullModel = (ComponentReceiveEventFullModel) XmlUtils.convertXmlStrToObject(ComponentReceiveEventFullModel.class, passiveReplyMsg);
        log.info("字符串参数转对象>>>>>> {}", JSON.toJSONString(componentReceiveEventFullModel));

        // 请求处理事件，并返回消息数据
        String resp = "success";

        // toUserName和appId不会同时存在
        // 微信推送关注用户触发的事件消息，可以进行回复
        // 请求处理事件，并返回消息数据
        try {
            log.info("请求处理被动消息>>>>>> componentEventDealUrl={} : componentReceiveEventFullModel={}", componentEventDealUrl, JSON.toJSONString(componentReceiveEventFullModel));
            String replyMsgData = HttpUtil.post(componentEventDealUrl, JSON.toJSONString(componentReceiveEventFullModel)); // 直接返回字符串有乱码问题
            log.info("接受处理过的被动消息<<<<<< componentReceiveEventFullModel={}", JSON.toJSONString(replyMsgData));
            Result<String> msgData = JSONObject.parseObject(replyMsgData, new TypeReference<Result<String>>(){});
            resp = msgData.getData();
        } catch (Exception e) {
            log.info("请求处理事件异常");
            e.printStackTrace();
            return "";
        }
        log.info("返回数据>>>>>> resp={}", JSON.toJSONString(resp));

        if (StrUtil.isBlank(resp)) {
            return "";
        }

        // 推送数据加密
        if (StrUtil.isNotBlank(encodingAesKey)) {
            try {
                resp = AesUtils.encryptMsg(token, appid, encodingAesKey, resp);
            } catch (Exception e) {
                log.error("消息加密异常>>>>>> ", e.getMessage());
                e.printStackTrace();
            }
        }
        return resp;
    }


    @GetMapping("/empower")
    @ApiOperation(value = "授权接口", response = String.class)
    public void empower(HttpServletResponse response) {
        log.info("/sys/healthy/wechat/msg/third/empower >>>>>>");
        try {
            response.sendRedirect("https://mp.weixin.qq.com/cgi-bin/componentloginpage?component_appid=wxb044d310a603d404&pre_auth_code=preauthcode@@@zPVJDrHpEpd2ZMVEh8l8a23PMw4K_5uIpJIH_Y6ugdadTmBzJcz8ZADiPnLZggMI2uKvsZLuxmMLm29ePVjLFQ&redirect_uri=https://dev1.cloud-seal.com/healthy-frontend/sys/healthy/wechat/third/componentloginpage&auth_type=1");
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @GetMapping("/componentloginpage")
    @ApiOperation(value = "授权回调接口", response = String.class)
    public String componentloginpage(@RequestParam("auth_code") String authCode, @RequestParam("expires_in") Integer expiresIn) {
        log.info("/sys/healthy/wechat/msg/third/componentloginpage >>>>>> authCode={}, expiresIn={}", authCode, expiresIn);
        return "success";
    }
}
